import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import com.jinsight.jetchart.*;


/**
 * This demo  generates  a  png  file  from  a  bar  chart  and  a client-side image map. After
 * clicking  the  'save png'  button,  a  png  file  is  created  in  the   current  directory
 * together  with  an  html  file.  See the method createSerieMap() below for further details.
 * Png file  generation  is useful if  charts are generated from  servlet requests, as well as
 * image maps. The  png file  may be created  in the server-side by a servlet, which generates
 * html code containing the image map and a reference to the mapped png file.
 * For this application  to work properly, the file 'Encoders.jar', found  in  the JetChart/lib
 * directory, must be included in the classpath.
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */

public class Main extends JFrame implements ActionListener,Runnable {

    Graph graph;
    BarSerie bs;
    JButton png;
    JLabel message;

    // This class is found in the Encoders.jar file.
    ChartEncoder chartEncoder;

    /**
     * A File object is created here. The  png encoding process generates
     * a file in the application directory with the name barchart.png.
     */
    File imageFile=new File("barchart.png");

    int maximum;


    public Main() {

	setTitle("The JetChart Library - Png file and image map generation");

	graph=new Graph();
	graph.setTitle(new String[]{"The JetChart Library","Png file and image map generation"});

	// Creates chart encoder. The chart encoder wraps a GIF, a PNG and a JPEG encoders.
	// To start encoding chart, some methods must be invoked on this object. See the 
	// run() method.
	chartEncoder=new ChartEncoder(graph);

	// Sets graphic context properties
	setChartProperties();

	// Creates bar series and sets its properties
	createBarSerie();
	
	// Adds bar series to the graphic context
	graph.addSerie(bs);
	
	Container ct=getContentPane();

	ct.add("Center",graph);

	// Creates top panel with a png generation button
	createTopPanel();

	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent evt) {
		System.exit(0);
	    }
	});
	

	setSize(550,350);

	// Centers frame on the window
	Dimension d=getToolkit().getScreenSize();
	setLocation((d.width-getSize().width)/2,(d.height-getSize().height)/2);

	setVisible(true);
    }
    
    private void setChartProperties() {

	// Enables 3D
	graph.set3DEnabled(true);

	graph.setHorizontalGraphEnabled(true);
	// Legends of each slice
	String[] labels={"Javasoft","Jinsight","Javalobby","Javacat","Javashareware","Jars"};
	graph.setLabels(labels);

	// Enables dragging
	graph.setDraggingEnabled(true);
	
	graph.getGraphSet(0).getGrid().setEnabled(true);
	graph.getGraphSet(0).getGrid().setColor(Color.lightGray);

    }

    private void createBarSerie() {
	// Values of bar
	double[] values={100,50,90,40,200,60};

	// Creates bar series
 	bs=new BarSerie(values,"Bar series title");
	bs.setColor(Color.green.darker());

    }

    private void createTopPanel() {
    
	JPanel topPanel=new JPanel();
	
	topPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	
	png=new JButton("Save png file");
	png.addActionListener(this);
	topPanel.add(png);
	
	message=new JLabel("Generating png file...");
	topPanel.add(message);
	message.setVisible(false);

	Container ct=getContentPane();
	ct.add("North",topPanel);
    }

    public void createSerieMap() {

	/**
	 * To generate a .map file, disable the lines marked with an asterisk
	 * and change the name of the file to be created  to another one with
	 * a .map extension. Change the map type to AbstractSerie.SERVER_SIDE_NCSA or
	 * AbstractSerie.SERVER_SIDE_CERN.
	 *
	 * In this example an html file is generated with a bar chart image mapped.
	 *  In a production environment, the ideal solution is a servlet with JetChart
	 * creating png images from client requests. The image map is loaded by a
	 * client browser, containing an image tag referring a servlet that generates
	 * the mapped image and send it back to the client browser on a servlet output
	 * stream.
	 */
	String serieMap="";
	// Creates an html text to insert series map. 
	serieMap+="<html><body bgcolor=white><br><br>\n\n"; // (*)

	serieMap+="<map name='map'>\n"; //(*)
	
	// Gets bar series image map.
	serieMap+=bs.getSerieMap(new String[]{"http://www.javasoft.com","http://www.jinsight.com",
					      "http://www.javalobby.com","http://www.javacat.com",
					      "http://www.javashareware.com","http://www.jars.com"},
				 AbstractSerie.CLIENT_SIDE,null)+"\n";
    
	
	serieMap+="</map>\n"; // (*)
	
	// Links barchart.png(the png image generated in this example) to series map.
	serieMap+="<br><br><image src=barchart.png usemap='#map' border=0>\n";  // (*)

	// Ends html code.
	serieMap+="<br><br></body></html>"; // (*)

	// Writes html code to a file.
	try {
	    FileWriter fw=new FileWriter("barchart.html");
	    fw.write(serieMap);
	    fw.close();
	}
	catch (IOException e) {
	    e.printStackTrace();
	}

    }

   

    public void actionPerformed(ActionEvent evt) {

	message.setVisible(true);
	message.getParent().validate();

	createSerieMap();
	
	/**
	 * Disables chart to prevent bar chart dragging while generating png. 
	 */
	graph.setEnabled(false);

	png.setEnabled(false);
	
	Thread t=new Thread(this);
	t.start();
	
    }

    public void run() {

	/**
	 * Starts encoding process. Set level compression to 5. It ranges
	 * from 1 to 9.
	 */
	try {
	    chartEncoder.pngEncode(imageFile,5);

	}
	catch (IOException e) {
	    e.printStackTrace();
	}
	finally {
	    // Enables chart.
	    graph.setEnabled(true);
	    message.setVisible(false);
	    png.setEnabled(true);
	}

    }

    public static void main(String[] args) {
	new Main();
    }

}
    
